/* HUB ISP - Program ATMEGA328P using only a USB HUB and simple circuitry
 * Copyright 2010, Paul Stoffregen <paul@pjrc.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "headers.h"


int main()
{
	int n, i, sig[3], addr, page, offset;
	unsigned char buf[PAGE_SIZE];

	open_usb(VENDOR_ID, PRODUCT_ID);
	printf("Programming enable: ");
	fflush(stdout);
	n = programming_enable();
	printf("0x%02X\n", n);
	if (n != 0x53) {
		printf("Communication is out of sync.  Please reset\n");
		printf("the AVR chip and then run this program again\n");
		printf("\nIf problems persist, try \"hub_test\" to verify\n");
		printf("your circuitry is connected properly\n");
		exit(1);
	}
	printf("Signature bytes:");
	fflush(stdout);
	for (i=0; i<3; i++) {
		sig[i] = read_signature_byte(i);
		printf(" 0x%02X", sig[i]);
		fflush(stdout);
	}
	printf("\n");

	n = read_intel_hex(HEX_FILE);
	if (n < 0) die("error while reading intel hex file %s\n", HEX_FILE);
	printf("Read \"%s\", %d bytes\n", HEX_FILE, n);

        printf("Erase Chip:\n");
	erase_chip();

        printf("Program Chip:\n");
	for (page=0; page<NUM_PAGES; page++) {
		addr = page * PAGE_SIZE;
		if (!ihex_bytes_within_range(addr, addr + PAGE_SIZE - 1)) continue;
		ihex_get_data(addr, PAGE_SIZE, buf);
		printf("  page %d:", page);
		fflush(stdout);
		for (offset=0; offset < (PAGE_SIZE/2); offset++) {
			write_word(offset, buf[offset * 2], buf[offset * 2 + 1]);
			printf(".");
			fflush(stdout);
		}
		write_page(page * PAGE_SIZE / 2);
		printf("\n");
	}

	printf("Fuse LSB\n");
	n = write_fuse_lsb(FUSE_LSB);
	if (n < 0) die("error writing Fuse LSB\n");
	printf("Fuse MSB\n");
	n = write_fuse_msb(FUSE_MSB);
	if (n < 0) die("error writing Fuse MSB\n");
	printf("Fuse EXT\n");
	n = write_fuse_ext(FUSE_EXT);
	if (n < 0) die("error writing Fuse EXT\n");
	printf("Lock Bits\n");
	n = write_lock_bits(LOCK_BITS);
	if (n < 0) die("error writing Lock Bits\n");
	printf("finished! :-)\n");
	return 0;
}


