/* HUB ISP - Program ATMEGA328P using only a USB HUB and simple circuitry
 * Copyright 2010, Paul Stoffregen <paul@pjrc.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "headers.h"


int main()
{
	int i, err;

	printf("\nTest #1: open HUB with vid:pid = %04x:%04x\n", VENDOR_ID, PRODUCT_ID);
	open_usb(VENDOR_ID, PRODUCT_ID);
	printf("  pass\n");

	printf("\nTest #2: check pullup resistors on ports 1, 2, 3\n");
	printf("  port1: ");
	printf(read_port(1) ? "pass\n" : "fail\n");
	printf("  port2: ");
	printf(read_port(2) ? "pass\n" : "fail\n");
	printf("  port3: ");
	printf(read_port(3) ? "pass\n" : "fail\n");

	printf("\nTest #3: Toggle MOSI, read back on MISO\n");
	printf("         (MOSI and MISO must be connected together)\n");
	printf("         LED connected to MOSI should blink slowly during this test\n");
	for (i=err=0; i<10; i++) {
		pulse_port(1);
		if (read_port(4) != 0) err = 1;
		printf(".");
		fflush(stdout);
		delay(0.3);
		pulse_port(2);
		if (read_port(4) != 1) err = 1;
		printf(".");
		fflush(stdout);
		delay(0.3);
	}
	if (!err) {
		printf("\n  pass: MISO correctly read back MOSI correctly\n");
	} else {
		printf("\n  fail: MISO did not read MOSI (are they connected together?)\n");
	}

	printf("\nTest #4: Blink SCLK, while holding MOSI high\n");
	printf("         LED connected to MOSI should remain on\n");
	printf("         LED connected to SCLK should flash briefly\n");
	pulse_port(1);
	for (i=err=0; i<20; i++) {
		pulse_port(3);
		if (read_port(4) != 0) err = 1;
		printf(".");
		fflush(stdout);
		delay(0.30);
	}
	if (!err) {
		printf("\n  pass: MISO correctly read back MOSI correctly\n");
	} else {
		printf("\n  fail: MISO did not read MOSI (are they connected together?)\n");
	}

	printf("\nTest #5: Blink SCLK, while holding MOSI low\n");
	printf("         LED connected to MOSI should remain off\n");
	printf("         LED connected to SCLK should flash briefly\n");
	pulse_port(2);
	for (i=err=0; i<20; i++) {
		pulse_port(3);
		if (read_port(4) != 1) err = 1;
		printf(".");
		fflush(stdout);
		delay(0.30);
	}
	if (!err) {
		printf("\n  pass: MISO correctly read back MOSI correctly\n");
	} else {
		printf("\n  fail: MISO did not read MOSI (are they connected together?)\n");
	}

	close_usb();
	return 0;
}


